window.Gleam.initSearch([{"doc":"kielet","title":"kielet","content":"# Kielet - GNU Gettext for Gleam\n\n[![Package Version](https://img.shields.io/hexpm/v/kielet)](https://hex.pm/packages/kielet)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/kielet/)\n\nKielet is a [GNU Gettext](https://www.gnu.org/software/gettext/) implementation for Gleam. With Kielet,\nyou can translate your Gleam or BEAM application without needing to change its source code or recompile\nit. Kielet implements translation functions for singular and plural forms and an MO file parser to read\ncompiled translations. Gleam source code can be processed with the `xgettext` program to automatically\ngenerate the translation templates (POT files).\n\n## Usage\n\nFirst add Kielet to your project:\n\n```sh\ngleam add kielet\n```\n\nThen you can start annotating your source code to prepare it for translation (skip the file reading and\nlanguage loading when you don&#39;t have languages yet):\n\n```gleam\nimport gleam/io\nimport kielet.{gettext as g_, ngettext as n_}\nimport kielet/context.{Context}\nimport kielet/database\nimport kielet/language\nimport simplifile\n\npub fn main() {\n  // This example uses simplifile to read the MO data\n  let assert Ok(mo_data) = simplifile.read_bits(&quot;./path/to/fi.mo&quot;)\n\n  // Load language from MO file\n  let assert Ok(finnish) = language.load(&quot;fi&quot;, mo_data)\n\n  // Create language database\n  let db = database.new() |&gt; database.add_language(finnish)\n\n  // Create translation context to choose active language\n  let ctx = Context(db, &quot;fi&quot;)\n\n  // &quot;Morjens, maailma!&quot;\n  io.println(\n    g_(ctx, &quot;Hello, world!&quot;)\n  )\n\n  // &quot;Ou jee, mulla on %s euroa&quot; -- The correct plural form is chosen based on the amount provided\n  io.println(\n    n_(ctx, &quot;Nice, I have %s euro&quot;, &quot;Nice, I have %s euros&quot;, 15)\n  )\n}\n```\n\nWhen you have annotated your code, but have no translations, the original messages will be returned.\nThus it&#39;s safe to add new strings to your program, the worst that can happen is that the original\nstrings will be shown.\n\nTo start translating your program, first extract the strings from your source code. You should read\nthe documentation of the `xgettext` program to do this, but here is an example command that works for\nGleam source code, when you have imported the functions as `g_` and `n_`:\n\n```sh\nxgettext src/**/*.gleam --keyword=g_:2 --keyword=n_:2,3 -o - --copyright-holder=&#39;Your Name&#39; --package-name=&#39;Your App&#39; --package-version=&#39;X.Y.Z&#39; --msgid-bugs-address=&#39;email@example.com&#39;\n```\n\nStore this output into a template file with the `.pot` extension and then use a translation app such as\n[Poedit](https://poedit.net) to create language specific translation files. The output should be human\nreadable PO files that you can add to your version tracking if wanted, and binary MO files that Kielet\ncan read.\n\n### String replacement\n\nNote that Kielet does not do any string replacement. It&#39;s convention to use `%s` to denote the number\nof items in a translatable string, but it&#39;s not enforced and has no special meaning. This means that\nthe translated message will contain the `%s` unchanged, and it is up to you to replace it with the\nappropriate number, taking into account the target locale&#39;s number format.\n\n## Gettext limitations\n\nDue to how Gettext is built, the source language (the language used in your source files) can only be a\nlanguage with two plural forms. This means that there is a singular form, used when there is one item,\nand a plural form that is used for every other amount of items. Unfortunately Gettext does not support\nlanguages with more plural forms such as Arabic as the source language. They are supported as\ntranslation targets, though.\n\nIf you do not have any usage of `ngettext` in your code, your source language can be any language. This\nmay be a risky choice, though, since it&#39;s quite likely you will need plural forms in the future.\n\n## Development\n\n```sh\ngleam test  # Run the tests\n```\n","url":"index.html"},{"doc":"kielet/mo","title":"Endianness","content":"pub type Endianness {\n  BigEndian\n  LittleEndian\n}\n\nBigEndian\n\n\nLittleEndian\n\n\nSynonyms:\nkielet/mo.Endianness\nkielet/mo Endianness","url":"kielet/mo.html#Endianness"},{"doc":"kielet/mo","title":"Header","content":"pub type Header {\n  Header(\n    revision: Revision,\n    string_count: Int,\n    og_table_offset: Int,\n    trans_table_offset: Int,\n    ht_size: Int,\n    ht_offset: Int,\n  )\n}\n\nHeader(\n  revision: Revision,\n  string_count: Int,\n  og_table_offset: Int,\n  trans_table_offset: Int,\n  ht_size: Int,\n  ht_offset: Int,\n)\n\nstring_count\n&lt;p&gt;Amount of strings in file.&lt;/p&gt;\n\nog_table_offset\n&lt;p&gt;Offset to table of strings in original language.&lt;/p&gt;\n\ntrans_table_offset\n&lt;p&gt;Offset to translations table.&lt;/p&gt;\n\nht_size\n&lt;p&gt;Hash table size (not used by this parser).&lt;/p&gt;\n\nht_offset\n&lt;p&gt;Hash table offset.&lt;/p&gt;\n\nSynonyms:\nkielet/mo.Header\nkielet/mo Header","url":"kielet/mo.html#Header"},{"doc":"kielet/mo","title":"MetaData","content":"pub type MetaData =\n  Dict(String, String)\n An MO file can contain metadata that looks quite like HTTP headers. The most\n important data here are the plural forms with the key `Plural-Forms`.\n\nSynonyms:\nkielet/mo.MetaData\nkielet/mo MetaData","url":"kielet/mo.html#MetaData"},{"doc":"kielet/mo","title":"Mo","content":"pub type Mo {\n  Mo(\n    endianness: Endianness,\n    header: Header,\n    translations: Translations,\n    metadata: MetaData,\n  )\n}\n\nMo(\n  endianness: Endianness,\n  header: Header,\n  translations: Translations,\n  metadata: MetaData,\n)\n\n\nSynonyms:\nkielet/mo.Mo\nkielet/mo Mo","url":"kielet/mo.html#Mo"},{"doc":"kielet/mo","title":"MoString","content":"pub type MoString {\n  Singular(context: String, content: String)\n  Plural(context: String, content: Dict(Int, String))\n}\n A translation in the file. The context is a free-form text that can be\n added as extra context for the translators.\nSingular(context: String, content: String)\n\n\nPlural(context: String, content: Dict(Int, String))\n\n\nSynonyms:\nkielet/mo.MoString\nkielet/mo MoString","url":"kielet/mo.html#MoString"},{"doc":"kielet/mo","title":"ParseError","content":"pub type ParseError {\n  MagicNumberNotFound\n  MalformedHeader\n  UnknownRevision(Revision)\n  OffsetPastEnd(Int)\n  MalformedOffsetTableEntry(BitArray)\n  StringNotUTF8(BitArray)\n  MetaItemMissing\n  MetaItemIsNotSingular\n}\n\nMagicNumberNotFound\n\n\nMalformedHeader\n\n\nUnknownRevision(Revision)\n\n\nOffsetPastEnd(Int)\n An offset was given that either pointed directly out of bounds or the data\n it pointed to would have been too big for the file.\n\nMalformedOffsetTableEntry(BitArray)\n\n\nStringNotUTF8(BitArray)\n\n\nMetaItemMissing\n Metadata is contained as a &quot;translation&quot; for the msgid &quot;&quot; (empty string).\n If this translation is missing, this error is returned.\n\nMetaItemIsNotSingular\n The metadata item is a plural.\n\nSynonyms:\nkielet/mo.ParseError\nkielet/mo ParseError","url":"kielet/mo.html#ParseError"},{"doc":"kielet/mo","title":"Revision","content":"pub type Revision {\n  Revision(major: Int, minor: Int)\n}\n\nRevision(major: Int, minor: Int)\n\n\nSynonyms:\nkielet/mo.Revision\nkielet/mo Revision","url":"kielet/mo.html#Revision"},{"doc":"kielet/mo","title":"Translations","content":"pub type Translations =\n  Dict(String, MoString)\n Dictionary of translations keyed by the original msgid.\n\nSynonyms:\nkielet/mo.Translations\nkielet/mo Translations","url":"kielet/mo.html#Translations"},{"doc":"kielet/mo","title":"parse","content":"pub fn parse(mo: BitArray) -&gt; Result(Mo, ParseError)\n Parse given MO file data.\nSynonyms:\nkielet/mo.parse\nkielet/mo parse","url":"kielet/mo.html#parse"},{"doc":"kielet/mo","title":"kielet/mo","content":" An MO file parser.\n\n Both little and big endian files are supported.\n\n All strings must be UTF-8.","url":"kielet/mo.html"},{"doc":"kielet/plurals/ast","title":"Ast","content":"pub type Ast {\n  N\n  Integer(Int)\n  BinaryOperation(operator: BinOp, lvalue: Ast, rvalue: Ast)\n  If(condition: Ast, truthy: Ast, falsy: Ast)\n  Paren(Ast)\n}\n\nN\n Represents the input value\n\nInteger(Int)\n Integer literal\n\nBinaryOperation(operator: BinOp, lvalue: Ast, rvalue: Ast)\n\n\nIf(condition: Ast, truthy: Ast, falsy: Ast)\n\n\nParen(Ast)\n Parenthesised expression\n\nSynonyms:\nkielet/plurals/ast.Ast\nkielet/plurals/ast Ast","url":"kielet/plurals/ast.html#Ast"},{"doc":"kielet/plurals/ast","title":"BinOp","content":"pub type BinOp {\n  Equal\n  NotEqual\n  GreaterThan\n  GreaterThanOrEqual\n  LowerThan\n  LowerThanOrEqual\n  Remainder\n  And\n  Or\n}\n A binary operation (operation with two arguments)\nEqual\n\n\nNotEqual\n\n\nGreaterThan\n\n\nGreaterThanOrEqual\n\n\nLowerThan\n\n\nLowerThanOrEqual\n\n\nRemainder\n\n\nAnd\n\n\nOr\n\n\nSynonyms:\nkielet/plurals/ast.BinOp\nkielet/plurals/ast BinOp","url":"kielet/plurals/ast.html#BinOp"},{"doc":"kielet/plurals/ast","title":"kielet/plurals/ast","content":" Plural forms abstract syntax tree","url":"kielet/plurals/ast.html"},{"doc":"kielet/plurals/evaluator","title":"eval","content":"pub fn eval(ast: Ast, input: Int) -&gt; Int\n Evaluate the given syntax tree with the input, returning an integer value\n representing the index of the plural form to choose.\nSynonyms:\nkielet/plurals/evaluator.eval\nkielet/plurals/evaluator eval","url":"kielet/plurals/evaluator.html#eval"},{"doc":"kielet/plurals/evaluator","title":"kielet/plurals/evaluator","content":" Evaluation engine for the syntax tree","url":"kielet/plurals/evaluator.html"},{"doc":"kielet/plurals/syntax_error","title":"SyntaxError","content":"pub type SyntaxError {\n  SyntaxError(\n    line: Int,\n    column: option.Option(Int),\n    reason: String,\n  )\n}\n Error returned when the syntax in a plural forms string is invalid.\nSyntaxError(\n  line: Int,\n  column: option.Option(Int),\n  reason: String,\n)\n\n\nSynonyms:\nkielet/plurals/syntax_error.SyntaxError\nkielet/plurals/syntax_error SyntaxError","url":"kielet/plurals/syntax_error.html#SyntaxError"},{"doc":"kielet/plurals/syntax_error","title":"kielet/plurals/syntax_error","content":"","url":"kielet/plurals/syntax_error.html"},{"doc":"kielet/plurals/tokenizer","title":"Token","content":"pub type Token {\n  N\n  NPlurals\n  Plural\n  Equals\n  NotEquals\n  GreaterThanOrEquals\n  LowerThanOrEquals\n  GreaterThan\n  LowerThan\n  Assignment\n  Ternary\n  TernaryElse\n  Remainder\n  Or\n  And\n  Semicolon\n  LParen\n  RParen\n  End\n  Int(value: Int)\n}\n\nN\n\n\nNPlurals\n\n\nPlural\n\n\nEquals\n\n\nNotEquals\n\n\nGreaterThanOrEquals\n\n\nLowerThanOrEquals\n\n\nGreaterThan\n\n\nLowerThan\n\n\nAssignment\n\n\nTernary\n\n\nTernaryElse\n\n\nRemainder\n\n\nOr\n\n\nAnd\n\n\nSemicolon\n\n\nLParen\n\n\nRParen\n\n\nEnd\n\n\nInt(value: Int)\n\n\nSynonyms:\nkielet/plurals/tokenizer.Token\nkielet/plurals/tokenizer Token","url":"kielet/plurals/tokenizer.html#Token"},{"doc":"kielet/plurals/tokenizer","title":"tokenize","content":"pub fn tokenize(\n  str: String,\n) -&gt; Result(\n  List(Token(kielet/plurals/tokenizer.Token)),\n  SyntaxError,\n)\n Tokenize the given plural forms syntax.\n\n Whitespace is ignored and backslashes at the end of lines are removed.\nSynonyms:\nkielet/plurals/tokenizer.tokenize\nkielet/plurals/tokenizer tokenize","url":"kielet/plurals/tokenizer.html#tokenize"},{"doc":"kielet/plurals/tokenizer","title":"kielet/plurals/tokenizer","content":" The tokenizer converts the plural forms syntax into a list of tokens for\n later parsing.","url":"kielet/plurals/tokenizer.html"},{"doc":"kielet/plurals/parser","title":"parse","content":"pub fn parse(\n  input: List(Token(kielet/plurals/tokenizer.Token)),\n) -&gt; Result(\n  #(Int, Ast),\n  List(DeadEnd(kielet/plurals/tokenizer.Token, a)),\n)\n Parse the given list of tokens into an abstract syntax tree.\nSynonyms:\nkielet/plurals/parser.parse\nkielet/plurals/parser parse","url":"kielet/plurals/parser.html#parse"},{"doc":"kielet/plurals/parser","title":"kielet/plurals/parser","content":" Parser for the plural form token stream.\n\n The trailing semicolon in the format is optional.","url":"kielet/plurals/parser.html"},{"doc":"kielet/plurals","title":"LoadError","content":"pub type LoadError {\n  ParsingFailed(err: ParseError)\n  NoPluralFormsHeader\n}\n Error returned when loading the plural forms from an MO file.\nParsingFailed(err: ParseError)\n\n\nNoPluralFormsHeader\n\n\nSynonyms:\nkielet/plurals.LoadError\nkielet/plurals LoadError","url":"kielet/plurals.html#LoadError"},{"doc":"kielet/plurals","title":"ParseError","content":"pub type ParseError {\n  TokenizerError(err: syntax_error.SyntaxError)\n  ParserError(err: List(nibble.DeadEnd(tokenizer.Token, Nil)))\n}\n Error returned when tokenizing or parsing the plural forms.\nTokenizerError(err: syntax_error.SyntaxError)\n\n\nParserError(err: List(nibble.DeadEnd(tokenizer.Token, Nil)))\n\n\nSynonyms:\nkielet/plurals.ParseError\nkielet/plurals ParseError","url":"kielet/plurals.html#ParseError"},{"doc":"kielet/plurals","title":"Plurals","content":"pub type Plurals {\n  Plurals(total: Int, algorithm: ast.Ast)\n}\n\nPlurals(total: Int, algorithm: ast.Ast)\n\n\nSynonyms:\nkielet/plurals.Plurals\nkielet/plurals Plurals","url":"kielet/plurals.html#Plurals"},{"doc":"kielet/plurals","title":"evaluate","content":"pub fn evaluate(plurals: Plurals, n: Int) -&gt; Int\n Evaluate given plurals, returning the index of the plural form to choose.\nSynonyms:\nkielet/plurals.evaluate\nkielet/plurals evaluate","url":"kielet/plurals.html#evaluate"},{"doc":"kielet/plurals","title":"load_from_mo","content":"pub fn load_from_mo(mo: Mo) -&gt; Result(Plurals, LoadError)\n Load plural forms from a parsed MO file.\nSynonyms:\nkielet/plurals.load_from_mo\nkielet/plurals load_from_mo","url":"kielet/plurals.html#load_from_mo"},{"doc":"kielet/plurals","title":"parse","content":"pub fn parse(input: String) -&gt; Result(Plurals, ParseError)\n Tokenize and parse given plural forms string.\nSynonyms:\nkielet/plurals.parse\nkielet/plurals parse","url":"kielet/plurals.html#parse"},{"doc":"kielet/plurals","title":"kielet/plurals","content":" Utilities for plural forms.","url":"kielet/plurals.html"},{"doc":"kielet/language","title":"Language","content":"pub opaque type Language\n\n\nSynonyms:\nkielet/language.Language\nkielet/language Language","url":"kielet/language.html#Language"},{"doc":"kielet/language","title":"LanguageLoadError","content":"pub type LanguageLoadError {\n  MoParseError(err: mo.ParseError)\n  PluralFormsLoadError(err: plurals.LoadError)\n}\n Error returned when an MO file is loaded.\nMoParseError(err: mo.ParseError)\n The MO file could not be parsed.\n\nPluralFormsLoadError(err: plurals.LoadError)\n The plural forms in the MO file were invalid or missing.\n\nSynonyms:\nkielet/language.LanguageLoadError\nkielet/language LanguageLoadError","url":"kielet/language.html#LanguageLoadError"},{"doc":"kielet/language","title":"TranslateError","content":"pub type TranslateError {\n  MsgIsSingular(String)\n  MsgIsPlural(String)\n  MsgNotFound(String)\n  PluralOutOfBounds(requested: Int, but_max_is: Int)\n}\n Error when trying to translate a message.\nMsgIsSingular(String)\n Tried to translate a plural message, but the translation is singular.\n\nMsgIsPlural(String)\n Tried to translate a singular message, but the translation is plural.\n\nMsgNotFound(String)\n There was no translation for the message.\n\nPluralOutOfBounds(requested: Int, but_max_is: Int)\n The plural algorithm returned a form that is out of bounds for the amount\n of plural forms.\n\nSynonyms:\nkielet/language.TranslateError\nkielet/language TranslateError","url":"kielet/language.html#TranslateError"},{"doc":"kielet/language","title":"get_code","content":"pub fn get_code(lang: Language) -&gt; String\n Get the language&#39;s language code.\nSynonyms:\nkielet/language.get_code\nkielet/language get_code","url":"kielet/language.html#get_code"},{"doc":"kielet/language","title":"get_plural_translation","content":"pub fn get_plural_translation(\n  lang: Language,\n  msgid: String,\n  n: Int,\n) -&gt; Result(String, TranslateError)\n Translate a plural message.\nSynonyms:\nkielet/language.get_plural_translation\nkielet/language get_plural_translation","url":"kielet/language.html#get_plural_translation"},{"doc":"kielet/language","title":"get_singular_translation","content":"pub fn get_singular_translation(\n  lang: Language,\n  msgid: String,\n) -&gt; Result(String, TranslateError)\n Translate a singular message.\nSynonyms:\nkielet/language.get_singular_translation\nkielet/language get_singular_translation","url":"kielet/language.html#get_singular_translation"},{"doc":"kielet/language","title":"load","content":"pub fn load(\n  code: String,\n  mo_file: BitArray,\n) -&gt; Result(Language, LanguageLoadError)\n Load a language from the given MO file.\nSynonyms:\nkielet/language.load\nkielet/language load","url":"kielet/language.html#load"},{"doc":"kielet/language","title":"kielet/language","content":" A language contains translations and plural form information, and can be\n used to translate messages.","url":"kielet/language.html"},{"doc":"kielet/database","title":"Database","content":"pub opaque type Database\n\n\nSynonyms:\nkielet/database.Database\nkielet/database Database","url":"kielet/database.html#Database"},{"doc":"kielet/database","title":"add_language","content":"pub fn add_language(db: Database, lang: Language) -&gt; Database\n Add a language to the database.\nSynonyms:\nkielet/database.add_language\nkielet/database add_language","url":"kielet/database.html#add_language"},{"doc":"kielet/database","title":"new","content":"pub fn new() -&gt; Database\n Create a new empty database.\nSynonyms:\nkielet/database.new\nkielet/database new","url":"kielet/database.html#new"},{"doc":"kielet/database","title":"translate_plural","content":"pub fn translate_plural(\n  db: Database,\n  msgid: String,\n  plural: String,\n  n: Int,\n  language_code: String,\n) -&gt; String\n Translate a plural message using the database.\n\n If the language is not found, does not have a translation for the message,\n or does not have the correct plural for the given `n`, the given plural\n message is returned.\nSynonyms:\nkielet/database.translate_plural\nkielet/database translate_plural","url":"kielet/database.html#translate_plural"},{"doc":"kielet/database","title":"translate_singular","content":"pub fn translate_singular(\n  db: Database,\n  msgid: String,\n  language_code: String,\n) -&gt; String\n Translate a singular message using the database.\n\n If the language is not found or does not have a translation for the message,\n the message is returned as-is.\nSynonyms:\nkielet/database.translate_singular\nkielet/database translate_singular","url":"kielet/database.html#translate_singular"},{"doc":"kielet/database","title":"kielet/database","content":" The database contains all the loaded languages.","url":"kielet/database.html"},{"doc":"kielet/context","title":"Context","content":"pub type Context {\n  Context(database: database.Database, language: String)\n}\n Translation context, containing both the database and the active language\n in one type for convenience.\nContext(database: database.Database, language: String)\n\n\nSynonyms:\nkielet/context.Context\nkielet/context Context","url":"kielet/context.html#Context"},{"doc":"kielet/context","title":"kielet/context","content":"","url":"kielet/context.html"},{"doc":"kielet","title":"gettext","content":"pub fn gettext(context: Context, msgid: String) -&gt; String\n Translate the given singular message.\nSynonyms:\nkielet.gettext\nkielet gettext","url":"kielet.html#gettext"},{"doc":"kielet","title":"ngettext","content":"pub fn ngettext(\n  context: Context,\n  singular: String,\n  plural: String,\n  n: Int,\n) -&gt; String\n Translate the given plural message. `n` is the amount of countable items\n in the message. For example for the English language, from `&quot;%s bunny&quot;` and\n `&quot;%s bunnies&quot;`, the latter would be returned when `n` is anything except 1.\nSynonyms:\nkielet.ngettext\nkielet ngettext","url":"kielet.html#ngettext"},{"doc":"kielet","title":"kielet","content":"","url":"kielet.html"}]);